@@ -1,7 +1,7 @@
 /* LICENSE BEGIN
     This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2020 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2020 Simon Hausmann <simon.hausmann@sixtyfps.io>
+    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
+    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
 
     SPDX-License-Identifier: GPL-3.0-only
     This file is also available under commercial licensing terms.
@@ -91,7 +91,7 @@ impl LinearGradientBrush {
     }
     /// Returns the color stops of the linear gradient.
     /// The stops are sorted by positions.
-    pub fn stops<'a>(&'a self) -> impl Iterator<Item = &'a GradientStop> + 'a {
+    pub fn stops(&self) -> impl Iterator<Item = &GradientStop> {
         // skip the first fake stop that just contains the angle
         self.0.iter().skip(1)
     }
@@ -127,7 +127,7 @@ impl InterpolatedPropertyValue for Brush {
             }
             (Brush::SolidColor(col), Brush::LinearGradient(grad)) => {
                 let mut new_grad = grad.clone();
-                for x in new_grad.0.as_slice_mut().iter_mut().skip(1) {
+                for x in new_grad.0.make_mut_slice().iter_mut().skip(1) {
                     x.color = col.interpolate(&x.color, t);
                 }
                 Brush::LinearGradient(new_grad)
@@ -140,7 +140,7 @@ impl InterpolatedPropertyValue for Brush {
                     Self::interpolate(target_value, self, 1. - t)
                 } else {
                     let mut new_grad = lhs.clone();
-                    let mut iter = new_grad.0.as_slice_mut().iter_mut();
+                    let mut iter = new_grad.0.make_mut_slice().iter_mut();
                     {
                         let angle = &mut iter.next().unwrap().position;
                         *angle = angle.interpolate(&rhs.angle(), t);
@@ -161,6 +161,7 @@ impl InterpolatedPropertyValue for Brush {
 }
 
 #[test]
+#[allow(clippy::float_cmp)] // We want bit-wise equality here
 fn test_linear_gradient_encoding() {
     let stops: SharedVector<GradientStop> = [
         GradientStop { position: 0.0, color: Color::from_argb_u8(255, 255, 0, 0) },
