@@ -38,6 +38,7 @@ use crate::rtti::*;
 use crate::window::WindowRc;
 use crate::{Callback, Property, SharedString};
 use const_field_offset::FieldOffsets;
+use core::cell::Cell;
 use core::pin::Pin;
 use sixtyfps_corelib_macros::*;
 use vtable::*;
@@ -355,6 +356,8 @@ pub struct TouchArea {
     pub pointer_event: Callback<PointerEventArg>,
     /// FIXME: remove this
     pub cached_rendering_data: CachedRenderingData,
+    /// true when we are currently grabbing the mouse
+    grabbed: Cell<bool>,
 }
 
 impl Item for TouchArea {
@@ -401,18 +404,20 @@ impl Item for TouchArea {
         if !self.enabled() {
             return InputEventResult::EventIgnored;
         }
-        let result =
-            if let MouseEvent::MouseReleased { pos, button: PointerEventButton::left } = event {
-                if euclid::rect(0., 0., self.width(), self.height()).contains(pos) {
-                    Self::FIELD_OFFSETS.clicked.apply_pin(self).call(&());
-                }
-                InputEventResult::EventAccepted
-            } else {
-                InputEventResult::GrabMouse
-            };
+        let result = if let MouseEvent::MouseReleased { pos, button } = event {
+            if button == PointerEventButton::left
+                && euclid::rect(0., 0., self.width(), self.height()).contains(pos)
+            {
+                Self::FIELD_OFFSETS.clicked.apply_pin(self).call(&());
+            }
+            InputEventResult::EventAccepted
+        } else {
+            InputEventResult::GrabMouse
+        };
 
         match event {
             MouseEvent::MousePressed { pos, button } => {
+                self.grabbed.set(true);
                 if button == PointerEventButton::left {
                     Self::FIELD_OFFSETS.pressed_x.apply_pin(self).set(pos.x);
                     Self::FIELD_OFFSETS.pressed_y.apply_pin(self).set(pos.y);
@@ -425,22 +430,25 @@ impl Item for TouchArea {
             }
             MouseEvent::MouseExit => {
                 Self::FIELD_OFFSETS.pressed.apply_pin(self).set(false);
-                Self::FIELD_OFFSETS.pointer_event.apply_pin(self).call(&(PointerEvent {
-                    button: PointerEventButton::none,
-                    kind: PointerEventKind::down,
-                },));
+                if self.grabbed.replace(false) {
+                    Self::FIELD_OFFSETS.pointer_event.apply_pin(self).call(&(PointerEvent {
+                        button: PointerEventButton::none,
+                        kind: PointerEventKind::cancel,
+                    },));
+                }
             }
             MouseEvent::MouseReleased { button, .. } => {
+                self.grabbed.set(false);
                 if button == PointerEventButton::left {
                     Self::FIELD_OFFSETS.pressed.apply_pin(self).set(false);
                 }
                 Self::FIELD_OFFSETS
                     .pointer_event
                     .apply_pin(self)
-                    .call(&(PointerEvent { button, kind: PointerEventKind::down },));
+                    .call(&(PointerEvent { button, kind: PointerEventKind::up },));
             }
             MouseEvent::MouseMoved { .. } => {
-                return if self.pressed() {
+                return if self.grabbed.get() {
                     Self::FIELD_OFFSETS.moved.apply_pin(self).call(&());
                     InputEventResult::GrabMouse
                 } else {
@@ -448,7 +456,7 @@ impl Item for TouchArea {
                 }
             }
             MouseEvent::MouseWheel { .. } => {
-                return if self.pressed() {
+                return if self.grabbed.get() {
                     InputEventResult::GrabMouse
                 } else {
                     InputEventResult::EventAccepted
@@ -1095,6 +1103,7 @@ pub struct WindowItem {
     pub height: Property<f32>,
     pub background: Property<Color>,
     pub title: Property<SharedString>,
+    pub no_frame: Property<bool>,
     pub icon: Property<crate::graphics::Image>,
     pub default_font_family: Property<SharedString>,
     pub default_font_size: Property<f32>,
@@ -1298,7 +1307,7 @@ impl Default for StandardButtonKind {
 }
 
 #[derive(
-    Copy, Clone, Eq, PartialEq, Hash, Debug, strum_macros::EnumString, strum_macros::ToString,
+    Copy, Clone, Eq, PartialEq, Hash, Debug, strum_macros::EnumString, strum_macros::Display,
 )]
 #[repr(C)]
 #[allow(non_camel_case_types)]
