@@ -1,7 +1,7 @@
 /* LICENSE BEGIN
     This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2020 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2020 Simon Hausmann <simon.hausmann@sixtyfps.io>
+    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
+    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
 
     SPDX-License-Identifier: GPL-3.0-only
     This file is also available under commercial licensing terms.
@@ -23,7 +23,7 @@ use crate::item_rendering::ItemRenderer;
 use crate::layout::{LayoutInfo, Orientation};
 #[cfg(feature = "rtti")]
 use crate::rtti::*;
-use crate::window::ComponentWindow;
+use crate::window::WindowRc;
 use crate::{Brush, Property};
 use const_field_offset::FieldOffsets;
 use core::pin::Pin;
@@ -44,6 +44,22 @@ impl Default for ImageFit {
     }
 }
 
+#[derive(
+    Copy, Clone, Debug, Eq, PartialEq, Hash, strum_macros::EnumString, strum_macros::Display,
+)]
+#[repr(C)]
+#[allow(non_camel_case_types)]
+pub enum ImageRendering {
+    smooth,
+    pixelated,
+}
+
+impl Default for ImageRendering {
+    fn default() -> Self {
+        ImageRendering::smooth
+    }
+}
+
 #[repr(C)]
 #[derive(FieldOffsets, Default, SixtyFPSElement)]
 #[pin]
@@ -55,11 +71,12 @@ pub struct ImageItem {
     pub width: Property<f32>,
     pub height: Property<f32>,
     pub image_fit: Property<ImageFit>,
+    pub image_rendering: Property<ImageRendering>,
     pub cached_rendering_data: CachedRenderingData,
 }
 
 impl Item for ImageItem {
-    fn init(self: Pin<&Self>, _window: &ComponentWindow) {}
+    fn init(self: Pin<&Self>, _window: &WindowRc) {}
 
     fn geometry(self: Pin<&Self>) -> Rect {
         euclid::rect(self.x(), self.y(), self.width(), self.height())
@@ -68,7 +85,7 @@ impl Item for ImageItem {
     fn layouting_info(
         self: Pin<&Self>,
         orientation: Orientation,
-        _window: &ComponentWindow,
+        _window: &WindowRc,
     ) -> LayoutInfo {
         let natural_size = self.source().size();
         LayoutInfo {
@@ -84,7 +101,7 @@ impl Item for ImageItem {
     fn input_event_filter_before_children(
         self: Pin<&Self>,
         _: MouseEvent,
-        _window: &ComponentWindow,
+        _window: &WindowRc,
         _self_rc: &ItemRc,
     ) -> InputEventFilterResult {
         InputEventFilterResult::ForwardAndIgnore
@@ -93,17 +110,17 @@ impl Item for ImageItem {
     fn input_event(
         self: Pin<&Self>,
         _: MouseEvent,
-        _window: &ComponentWindow,
+        _window: &WindowRc,
         _self_rc: &ItemRc,
     ) -> InputEventResult {
         InputEventResult::EventIgnored
     }
 
-    fn key_event(self: Pin<&Self>, _: &KeyEvent, _window: &ComponentWindow) -> KeyEventResult {
+    fn key_event(self: Pin<&Self>, _: &KeyEvent, _window: &WindowRc) -> KeyEventResult {
         KeyEventResult::EventIgnored
     }
 
-    fn focus_event(self: Pin<&Self>, _: &FocusEvent, _window: &ComponentWindow) {}
+    fn focus_event(self: Pin<&Self>, _: &FocusEvent, _window: &WindowRc) {}
 
     fn render(self: Pin<&Self>, backend: &mut &mut dyn ItemRenderer) {
         (*backend).draw_image(self)
@@ -128,6 +145,7 @@ pub struct ClippedImage {
     pub width: Property<f32>,
     pub height: Property<f32>,
     pub image_fit: Property<ImageFit>,
+    pub image_rendering: Property<ImageRendering>,
     pub colorize: Property<Brush>,
     pub source_clip_x: Property<i32>,
     pub source_clip_y: Property<i32>,
@@ -137,7 +155,7 @@ pub struct ClippedImage {
 }
 
 impl Item for ClippedImage {
-    fn init(self: Pin<&Self>, _window: &ComponentWindow) {}
+    fn init(self: Pin<&Self>, _window: &WindowRc) {}
 
     fn geometry(self: Pin<&Self>) -> Rect {
         euclid::rect(self.x(), self.y(), self.width(), self.height())
@@ -146,7 +164,7 @@ impl Item for ClippedImage {
     fn layouting_info(
         self: Pin<&Self>,
         orientation: Orientation,
-        _window: &ComponentWindow,
+        _window: &WindowRc,
     ) -> LayoutInfo {
         let natural_size = self.source().size();
         LayoutInfo {
@@ -162,7 +180,7 @@ impl Item for ClippedImage {
     fn input_event_filter_before_children(
         self: Pin<&Self>,
         _: MouseEvent,
-        _window: &ComponentWindow,
+        _window: &WindowRc,
         _self_rc: &ItemRc,
     ) -> InputEventFilterResult {
         InputEventFilterResult::ForwardAndIgnore
@@ -171,17 +189,17 @@ impl Item for ClippedImage {
     fn input_event(
         self: Pin<&Self>,
         _: MouseEvent,
-        _window: &ComponentWindow,
+        _window: &WindowRc,
         _self_rc: &ItemRc,
     ) -> InputEventResult {
         InputEventResult::EventIgnored
     }
 
-    fn key_event(self: Pin<&Self>, _: &KeyEvent, _window: &ComponentWindow) -> KeyEventResult {
+    fn key_event(self: Pin<&Self>, _: &KeyEvent, _window: &WindowRc) -> KeyEventResult {
         KeyEventResult::EventIgnored
     }
 
-    fn focus_event(self: Pin<&Self>, _: &FocusEvent, _window: &ComponentWindow) {}
+    fn focus_event(self: Pin<&Self>, _: &FocusEvent, _window: &WindowRc) {}
 
     fn render(self: Pin<&Self>, backend: &mut &mut dyn ItemRenderer) {
         (*backend).draw_clipped_image(self)
