@@ -14,7 +14,7 @@ When adding an item or a property, it needs to be kept in sync with different pl
 Lookup the [`crate::items`] module documentation.
 */
 
-use super::{Item, ItemConsts, ItemRc, PointArg, VoidArg};
+use super::{Item, ItemConsts, ItemRc, PointArg, PointerEventButton, VoidArg};
 use crate::graphics::{Brush, Color, FontRequest, Rect};
 use crate::input::{
     FocusEvent, InputEventResult, KeyEvent, KeyEventResult, KeyEventType, KeyboardModifiers,
@@ -120,12 +120,7 @@ impl Item for Text {
 
     fn layouting_info(self: Pin<&Self>, orientation: Orientation, window: &WindowRc) -> LayoutInfo {
         let implicit_size = |max_width| {
-            window.text_size(
-                &self.cached_rendering_data,
-                &|| self.unresolved_font_request(),
-                self.text().as_str(),
-                max_width,
-            )
+            window.text_size(self.unresolved_font_request(), self.text().as_str(), max_width)
         };
 
         // Stretch uses `round_layout` to explicitly align the top left and bottom right of layout nodes
@@ -135,16 +130,9 @@ impl Item for Text {
             Orientation::Horizontal => {
                 let implicit_size = implicit_size(None);
                 let min = match self.overflow() {
-                    TextOverflow::elide => implicit_size.width.min(
-                        window
-                            .text_size(
-                                &self.cached_rendering_data,
-                                &|| self.unresolved_font_request(),
-                                "…",
-                                None,
-                            )
-                            .width,
-                    ),
+                    TextOverflow::elide => implicit_size
+                        .width
+                        .min(window.text_size(self.unresolved_font_request(), "…", None).width),
                     TextOverflow::clip => match self.wrap() {
                         TextWrap::no_wrap => implicit_size.width,
                         TextWrap::word_wrap => 0.,
@@ -276,12 +264,17 @@ impl Item for TextInput {
     }
 
     fn layouting_info(self: Pin<&Self>, orientation: Orientation, window: &WindowRc) -> LayoutInfo {
+        let text = self.text();
         let implicit_size = |max_width| {
-            let text = self.text();
             window.text_size(
-                &self.cached_rendering_data,
-                &|| self.unresolved_font_request(),
-                if text.is_empty() { "*" } else { text.as_str() },
+                self.unresolved_font_request(),
+                {
+                    if text.is_empty() {
+                        "*"
+                    } else {
+                        text.as_str()
+                    }
+                },
                 max_width,
             )
         };
@@ -332,7 +325,7 @@ impl Item for TextInput {
             return InputEventResult::EventIgnored;
         }
         match event {
-            MouseEvent::MousePressed { pos } => {
+            MouseEvent::MousePressed { pos, button: PointerEventButton::left } => {
                 let clicked_offset = window.text_input_byte_offset_for_position(self, pos) as i32;
                 self.as_ref().pressed.set(true);
                 self.as_ref().anchor_position.set(clicked_offset);
@@ -341,9 +334,8 @@ impl Item for TextInput {
                     window.clone().set_focus_item(self_rc);
                 }
             }
-            MouseEvent::MouseReleased { .. } | MouseEvent::MouseExit => {
-                self.as_ref().pressed.set(false)
-            }
+            MouseEvent::MouseReleased { button: PointerEventButton::left, .. }
+            | MouseEvent::MouseExit => self.as_ref().pressed.set(false),
             MouseEvent::MouseMoved { pos } => {
                 if self.as_ref().pressed.get() {
                     let clicked_offset =
@@ -351,7 +343,7 @@ impl Item for TextInput {
                     self.set_cursor_position(clicked_offset, window);
                 }
             }
-            MouseEvent::MouseWheel { .. } => return InputEventResult::EventIgnored,
+            _ => return InputEventResult::EventIgnored,
         }
         InputEventResult::EventAccepted
     }
