@@ -11,7 +11,7 @@ LICENSE END */
 
 // cspell:ignore coord
 
-use crate::{slice::Slice, SharedVector};
+use crate::{items::DialogButtonRole, slice::Slice, SharedVector};
 
 /// Vertical or Horizontal orientation
 #[derive(Copy, Clone, Eq, PartialEq, Hash, Debug)]
@@ -719,6 +719,70 @@ pub fn solve_path_layout(data: &PathLayoutData, repeater_indexes: Slice<u32>) ->
     result
 }
 
+/// Given the cells of a layout of a Dialog, re-order the button according to the platform
+///
+/// This function assume that the `roles` contains the roles of the button which are the first `cells`
+/// It will simply change the column field of the cell
+pub fn reorder_dialog_button_layout(cells: &mut [GridLayoutCellData], roles: &[DialogButtonRole]) {
+    fn add_buttons(
+        cells: &mut [GridLayoutCellData],
+        roles: &[DialogButtonRole],
+        idx: &mut u16,
+        role: DialogButtonRole,
+    ) {
+        for (cell, r) in cells.iter_mut().zip(roles.iter()) {
+            if *r == role {
+                cell.col_or_row = *idx;
+                *idx += 1;
+            }
+        }
+    }
+
+    let mut idx = 0;
+
+    if cfg!(windows) {
+        add_buttons(cells, roles, &mut idx, DialogButtonRole::reset);
+        idx += 1;
+        add_buttons(cells, roles, &mut idx, DialogButtonRole::accept);
+        add_buttons(cells, roles, &mut idx, DialogButtonRole::action);
+        add_buttons(cells, roles, &mut idx, DialogButtonRole::reject);
+        add_buttons(cells, roles, &mut idx, DialogButtonRole::apply);
+        add_buttons(cells, roles, &mut idx, DialogButtonRole::help);
+    } else if cfg!(target_os = "macos") {
+        add_buttons(cells, roles, &mut idx, DialogButtonRole::help);
+        add_buttons(cells, roles, &mut idx, DialogButtonRole::reset);
+        add_buttons(cells, roles, &mut idx, DialogButtonRole::apply);
+        add_buttons(cells, roles, &mut idx, DialogButtonRole::action);
+        idx += 1;
+        add_buttons(cells, roles, &mut idx, DialogButtonRole::reject);
+        add_buttons(cells, roles, &mut idx, DialogButtonRole::accept);
+
+        // assume some unix check if XDG_CURRENT_DESKTOP stats with K
+    } else if std::env::var("XDG_CURRENT_DESKTOP")
+        .ok()
+        .and_then(|v| v.as_bytes().get(0).copied())
+        .map_or(false, |x| x.to_ascii_uppercase() == b'K')
+    {
+        // KDE variant
+        add_buttons(cells, roles, &mut idx, DialogButtonRole::help);
+        add_buttons(cells, roles, &mut idx, DialogButtonRole::reset);
+        idx += 1;
+        add_buttons(cells, roles, &mut idx, DialogButtonRole::action);
+        add_buttons(cells, roles, &mut idx, DialogButtonRole::accept);
+        add_buttons(cells, roles, &mut idx, DialogButtonRole::apply);
+        add_buttons(cells, roles, &mut idx, DialogButtonRole::reject);
+    } else {
+        // GNOME variant and fallback for WASM build
+        add_buttons(cells, roles, &mut idx, DialogButtonRole::help);
+        add_buttons(cells, roles, &mut idx, DialogButtonRole::reset);
+        idx += 1;
+        add_buttons(cells, roles, &mut idx, DialogButtonRole::action);
+        add_buttons(cells, roles, &mut idx, DialogButtonRole::apply);
+        add_buttons(cells, roles, &mut idx, DialogButtonRole::reject);
+        add_buttons(cells, roles, &mut idx, DialogButtonRole::accept);
+    }
+}
+
 #[cfg(feature = "ffi")]
 pub(crate) mod ffi {
     #![allow(unsafe_code)]
@@ -779,4 +843,16 @@ pub(crate) mod ffi {
     ) {
         *result = super::solve_path_layout(data, repeater_indexes)
     }
+
+    /// Calls [`reorder_dialog_button_layout`].
+    ///
+    /// Safety: `cells` must be a pointer to a mutable array of cell data, the array must have at
+    /// least `roles.len()` elements.
+    #[no_mangle]
+    pub unsafe extern "C" fn sixtyfps_reorder_dialog_button_layout(
+        cells: *mut GridLayoutCellData,
+        roles: Slice<DialogButtonRole>,
+    ) {
+        reorder_dialog_button_layout(core::slice::from_raw_parts_mut(cells, roles.len()), &roles);
+    }
 }
