@@ -31,11 +31,29 @@ pub struct ModelPeer {
     inner: Weak<RefCell<ModelPeerInner>>,
 }
 
+/// This trait defines the interface that users of a model can use to track changes
+/// to a model. It is supplied via [`Model::model_tracker`] and implementation usually
+/// return a reference to its field of [`ModelNotify`].
+pub trait ModelTracker {
+    /// Attach one peer. The peer will be notified when the model changes
+    fn attach_peer(&self, peer: ModelPeer);
+    /// Register the model as a dependency to the current binding being evaluated, so
+    /// that it will be notified when the model changes its size.
+    fn track_row_count_changes(&self);
+}
+
+impl ModelTracker for () {
+    fn attach_peer(&self, _peer: ModelPeer) {}
+
+    fn track_row_count_changes(&self) {}
+}
+
 /// Dispatch notifications from a [`Model`] to one or several [`ModelPeer`].
 /// Typically, you would want to put this in the implementation of the Model
 #[derive(Default)]
 pub struct ModelNotify {
     inner: RefCell<weak_table::PtrWeakHashSet<Weak<RefCell<ModelPeerInner>>>>,
+    model_dirty_property: RefCell<Option<Pin<Rc<Property<()>>>>>,
 }
 
 impl ModelNotify {
@@ -47,20 +65,44 @@ impl ModelNotify {
     }
     /// Notify the peers that rows were added
     pub fn row_added(&self, index: usize, count: usize) {
+        if let Some(model_dirty_tracker) = self.model_dirty_property.borrow().as_ref() {
+            model_dirty_tracker.mark_dirty()
+        }
         for peer in self.inner.borrow().iter() {
             peer.borrow_mut().row_added(index, count)
         }
     }
     /// Notify the peers that rows were removed
     pub fn row_removed(&self, index: usize, count: usize) {
+        if let Some(model_dirty_tracker) = self.model_dirty_property.borrow().as_ref() {
+            model_dirty_tracker.mark_dirty()
+        }
         for peer in self.inner.borrow().iter() {
             peer.borrow_mut().row_removed(index, count)
         }
     }
     /// Attach one peer. The peer will be notified when the model changes
+    #[deprecated(
+        note = "In your model, re-implement `model_tracker` of the `Model` trait instead of calling this function from our `attach_peer` implementation"
+    )]
     pub fn attach(&self, peer: ModelPeer) {
+        self.attach_peer(peer)
+    }
+}
+
+impl ModelTracker for ModelNotify {
+    /// Attach one peer. The peer will be notified when the model changes
+    fn attach_peer(&self, peer: ModelPeer) {
         peer.inner.upgrade().map(|rc| self.inner.borrow_mut().insert(rc));
     }
+
+    fn track_row_count_changes(&self) {
+        self.model_dirty_property
+            .borrow_mut()
+            .get_or_insert_with(|| Rc::pin(Default::default()))
+            .as_ref()
+            .get();
+    }
 }
 
 /// A Model is providing Data for the Repeater or ListView elements of the `.60` language
@@ -74,7 +116,7 @@ impl ModelNotify {
 /// As an example, let's see the implementation of [`VecModel`].
 ///
 /// ```
-/// # use sixtyfps_corelib::model::{Model, ModelNotify, ModelPeer};
+/// # use sixtyfps_corelib::model::{Model, ModelNotify, ModelPeer, ModelTracker};
 /// pub struct VecModel<T> {
 ///     // the backing data, stored in a `RefCell` as this model can be modified
 ///     array: std::cell::RefCell<Vec<T>>,
@@ -99,9 +141,8 @@ impl ModelNotify {
 ///         self.notify.row_changed(row);
 ///     }
 ///
-///     fn attach_peer(&self, peer: ModelPeer) {
-///         // simply forward to ModelNotify::attach
-///         self.notify.attach(peer);
+///     fn model_tracker(&self) -> &dyn ModelTracker {
+///         &self.notify
 ///     }
 ///
 ///     fn as_any(&self) -> &dyn core::any::Any {
@@ -150,7 +191,15 @@ pub trait Model {
         );
     }
     /// The implementation should forward to [`ModelNotify::attach`]
-    fn attach_peer(&self, peer: ModelPeer);
+    #[deprecated(note = "Re-implement model_tracker instead of this function")]
+    fn attach_peer(&self, peer: ModelPeer) {
+        self.model_tracker().attach_peer(peer)
+    }
+
+    /// The implementation should return a reference to its [`ModelNotify`] field.
+    fn model_tracker(&self) -> &dyn ModelTracker {
+        &()
+    }
 
     /// Returns an iterator visiting all elements of the model.
     fn iter(&self) -> ModelIterator<Self::Data>
@@ -264,8 +313,8 @@ impl<T: Clone + 'static> Model for VecModel<T> {
         self.notify.row_changed(row);
     }
 
-    fn attach_peer(&self, peer: ModelPeer) {
-        self.notify.attach(peer);
+    fn model_tracker(&self) -> &dyn ModelTracker {
+        &self.notify
     }
 
     fn as_any(&self) -> &dyn core::any::Any {
@@ -284,10 +333,6 @@ impl Model for usize {
         row as i32
     }
 
-    fn attach_peer(&self, _peer: ModelPeer) {
-        // The model is read_only: nothing to do
-    }
-
     fn as_any(&self) -> &dyn core::any::Any {
         self
     }
@@ -306,10 +351,6 @@ impl Model for bool {
 
     fn row_data(&self, _row: usize) -> Self::Data {}
 
-    fn attach_peer(&self, _peer: ModelPeer) {
-        // The model is read_only: nothing to do
-    }
-
     fn as_any(&self) -> &dyn core::any::Any {
         self
     }
@@ -375,12 +416,22 @@ impl<T> Model for ModelHandle<T> {
         self.0.as_ref().unwrap().row_data(row)
     }
 
+    fn set_row_data(&self, row: usize, data: Self::Data) {
+        self.0.as_ref().unwrap().set_row_data(row, data)
+    }
+
     fn attach_peer(&self, peer: ModelPeer) {
+        // Forward, in case the model doesn't provide `model_tracker` yet.
         if let Some(model) = self.0.as_ref() {
+            #[allow(deprecated)]
             model.attach_peer(peer);
         }
     }
 
+    fn model_tracker(&self) -> &dyn ModelTracker {
+        self.0.as_ref().map_or(&(), |model| model.model_tracker())
+    }
+
     fn as_any(&self) -> &dyn core::any::Any {
         self.0.as_ref().map_or(&(), |model| model.as_any())
     }
@@ -540,6 +591,7 @@ impl<C: RepeatedComponent + 'static> Repeater<C> {
             (*Rc::make_mut(&mut self.inner.borrow_mut()).get_mut()) = RepeaterInner::default();
             if let ModelHandle(Some(m)) = model.get() {
                 let peer: Rc<RefCell<dyn ViewAbstraction>> = self.inner.borrow().clone();
+                #[allow(deprecated)]
                 m.attach_peer(ModelPeer { inner: Rc::downgrade(&peer) });
             }
         }
@@ -642,7 +694,7 @@ impl<C: RepeatedComponent + 'static> Repeater<C> {
                     }
                 }
 
-                let element_height = if count.get() > 0 {
+                let mut element_height = if count.get() > 0 {
                     total_height.get() / (count.get() as f32)
                 } else {
                     // There seems to be currently no items. Just instantiate one item.
@@ -666,24 +718,41 @@ impl<C: RepeatedComponent + 'static> Repeater<C> {
 
                 let min_height = min_height.get().min(element_height).max(1.);
 
-                viewport_height.set(element_height * row_count as f32);
-                self.inner.borrow().borrow_mut().cached_item_height = element_height;
-                if -viewport_y.get() > element_height * row_count as f32 - listview_height {
-                    viewport_y.set(-(element_height * row_count as f32 - listview_height).max(0.))
+                let mut offset_y = -viewport_y.get().min(0.);
+                if offset_y > element_height * row_count as f32 - listview_height
+                    && offset_y > viewport_height.get() - listview_height
+                {
+                    offset_y = (element_height * row_count as f32 - listview_height).max(0.);
                 }
-                let offset = (-viewport_y.get_untracked() / element_height).floor() as usize;
-                let mut count =
-                    ((listview_height / min_height).ceil() as usize).min(row_count - offset);
+                let mut count = ((listview_height / min_height).ceil() as usize)
+                    // count never decreases to avoid too much flickering if items have different size
+                    .max(self.inner.borrow().borrow().components.len())
+                    .min(row_count);
+                let mut offset =
+                    ((offset_y / element_height).floor() as usize).min(row_count - count);
+                self.inner.borrow().borrow_mut().cached_item_height = element_height;
                 loop {
+                    self.inner.borrow().borrow_mut().cached_item_height = element_height;
                     self.set_offset(offset, count);
                     self.ensure_updated_impl(init, &model, count);
                     let end = self.compute_layout_listview(viewport_width, listview_width);
-                    let diff = listview_height - viewport_y.get_untracked() - end;
-                    if diff > 0. && count < row_count - offset {
-                        count = (count + (diff / element_height).ceil() as usize)
-                            .min(row_count - offset);
+                    let adjusted_element_height =
+                        (end - element_height * offset as f32) / count as f32;
+                    element_height = adjusted_element_height;
+                    let diff = listview_height + offset_y - end;
+                    if diff > 0.5 && count < row_count {
+                        // we did not create enough item, try increasing count until it matches
+                        count = (count + (diff / element_height).ceil() as usize).min(row_count);
+                        if offset + count > row_count {
+                            // apparently, we scrolled past the end, so decrease the offset and make offset_y
+                            // so we just are at the end
+                            offset = row_count - count;
+                            offset_y = (offset_y - diff).max(0.);
+                        }
                         continue;
                     }
+                    viewport_height.set((element_height * row_count as f32).max(end));
+                    viewport_y.set(-offset_y);
                     break;
                 }
             })
@@ -808,3 +877,40 @@ pub struct StandardListViewItem {
     /// The text content of the item
     pub text: crate::SharedString,
 }
+
+#[test]
+fn test_tracking_model_handle() {
+    let model: Rc<VecModel<u8>> = Rc::new(Default::default());
+    let handle = ModelHandle::new(model.clone());
+    let tracker = Box::pin(crate::properties::PropertyTracker::default());
+    assert_eq!(
+        tracker.as_ref().evaluate(|| {
+            handle.model_tracker().track_row_count_changes();
+            handle.row_count()
+        }),
+        0
+    );
+    assert!(!tracker.is_dirty());
+    model.push(42);
+    model.push(100);
+    assert!(tracker.is_dirty());
+    assert_eq!(
+        tracker.as_ref().evaluate(|| {
+            handle.model_tracker().track_row_count_changes();
+            handle.row_count()
+        }),
+        2
+    );
+    assert!(!tracker.is_dirty());
+    model.set_row_data(0, 41);
+    assert!(!tracker.is_dirty());
+    model.remove(0);
+    assert!(tracker.is_dirty());
+    assert_eq!(
+        tracker.as_ref().evaluate(|| {
+            handle.model_tracker().track_row_count_changes();
+            handle.row_count()
+        }),
+        1
+    );
+}
