@@ -1,7 +1,7 @@
 /* LICENSE BEGIN
     This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2020 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2020 Simon Hausmann <simon.hausmann@sixtyfps.io>
+    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
+    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
 
     SPDX-License-Identifier: GPL-3.0-only
     This file is also available under commercial licensing terms.
@@ -41,6 +41,11 @@ impl SharedString {
         self.inner.header.header
     }
 
+    /// Return true if the String is empty
+    pub fn is_empty(&self) -> bool {
+        self.len() == 0
+    }
+
     /// Return a slice to the string
     pub fn as_str(&self) -> &str {
         unsafe {
@@ -271,9 +276,15 @@ impl From<&String> for SharedString {
     }
 }
 
-impl Into<String> for SharedString {
-    fn into(self) -> String {
-        self.as_str().to_string()
+impl From<SharedString> for String {
+    fn from(s: SharedString) -> String {
+        s.as_str().to_string()
+    }
+}
+
+impl From<&SharedString> for String {
+    fn from(s: &SharedString) -> String {
+        s.as_str().to_string()
     }
 }
 
