@@ -1,7 +1,7 @@
 /* LICENSE BEGIN
     This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2020 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2020 Simon Hausmann <simon.hausmann@sixtyfps.io>
+    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
+    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
 
     SPDX-License-Identifier: GPL-3.0-only
     This file is also available under commercial licensing terms.
@@ -114,9 +114,9 @@ impl Timer {
 impl Drop for Timer {
     fn drop(&mut self) {
         if let Some(id) = self.id.get() {
-            CURRENT_TIMERS.with(|timers| {
+            let _ = CURRENT_TIMERS.try_with(|timers| {
                 timers.borrow_mut().remove_timer(id);
-            })
+            });
         }
     }
 }
@@ -286,7 +286,7 @@ impl TimerList {
 
 thread_local!(static CURRENT_TIMERS : RefCell<TimerList> = RefCell::default());
 
-fn lower_bound<T>(vec: &Vec<T>, mut less_than: impl FnMut(&T) -> bool) -> usize {
+fn lower_bound<T>(vec: &[T], mut less_than: impl FnMut(&T) -> bool) -> usize {
     let mut left = 0;
     let mut right = vec.len();
 
